/*
	Copyright (c) 2013, Raymond Dodge
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
		* Redistributions of source code must retain the above copyright
		  notice, this list of conditions and the following disclaimer.
		* Redistributions in binary form must reproduce the above copyright
		  notice, this list of conditions and the following disclaimer in the
		  documentation and/or other materials provided with the distribution.
		* Neither the name "<PRODUCT NAME>" nor the names of its contributors
		  may be used to endorse or promote products derived from this software
		  without specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
	DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
	ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
/*****************************************************************************
 * Methods relating to the printing of a stem-and-leaf plot
 * @author Raymond Dodge
*****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "stemAndLeaf.h"

/**
 *  returns 10 to the xth power.
 *  Same as the gcc exp10 function.
 */
double exp10(double x) {return exp(x * log(10));}




slData_t convertStringToSlData(const char* const str) {
	double value = atof(str);
	return convertDecimalToSlData(value);
}

slData_t convertDecimalToSlData(double value) {
	slData_t retVal;
	double power = floor( log10( value ) );
	
	retVal.power = (char) power;
	value = value + exp10(power - 4);
	
	retVal.leastSigDig1 = (char) floor(value / exp10(power));
	retVal.leastSigDig2 = (char) floor(value / exp10(power - 1)) % 10;
	retVal.leastSigDig3 = (char) ((int) floor(value / exp10(power - 2)) % 10);
	
	/*
	
	fprintf(stderr, "%lg = %d.%d%de%d; ",
		value,
		retVal.leastSigDig1,
		retVal.leastSigDig2,
		retVal.leastSigDig3,
		retVal.power);
	*/
						
	return retVal;
}

void printStemAndLeafPlot(
		const int valuec,
		const slData_t* const valuea,
		const graphTypes_t graphType
) {
	int i;
	char currentStem = -1;
	const char max_power = valuea[valuec - 1].power;
	const int twoDigitStem = (1 == valuea[valuec - 1].leastSigDig1);
	
	
	printf("\n---+---------   2|3 = %g", exp10(max_power) * 2.3 / (twoDigitStem ? 10 : 1)); 
	
	
	for (i = 0; i < valuec; i++) {
		const slData_t* const currentValue = (valuea + i);
		const char powerOffset = max_power - currentValue->power - (twoDigitStem ? 1 : 0 );
		char newStem, newLeaf;
		
		if (powerOffset == -1) {
			newStem = currentValue->leastSigDig1 * 10 + currentValue->leastSigDig2;
			newLeaf = currentValue->leastSigDig3;
		} else if (powerOffset == 0) {
			newStem = currentValue->leastSigDig1;
			newLeaf = currentValue->leastSigDig2;
		} else if (powerOffset == 1) {
			newStem = 0;
			newLeaf = currentValue->leastSigDig1;
		} else {
			newStem = 0;
			newLeaf = 0;
		}
		
		if (SPLIT_DECIMAL != graphType) {
			if (currentStem != newStem) {
				int i;
				for (i = currentStem + 1; i <= newStem; i++) {
					printf("\n%2i | ", i);
				}
				
				currentStem = newStem;
			}
		} else {
			int newStem2 = newStem * 2 + (newLeaf >= 5 ? 1 : 0);
			
			if (currentStem != newStem2) {
				int i;
				for (i = currentStem + 1; i <= newStem2; i++) {
					printf("\n%2i | ", i / 2);
				}
				
				currentStem = newStem2;
			}
		}
		printf("%i", newLeaf);
		
	}
	printf("\n");
}
